﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Leadtools;
using Leadtools.Multimedia;
using System.Threading;
using System.IO;
using System.Diagnostics;
using Leadtools.Demos;

namespace VideoToGIFLeadtools
{
   public partial class MainForm : Form
   {
      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main()
      {
         Support.SetLicense();
         Application.Run(new MainForm());
      }

      //initializing variables
      private ConvertCtrl convertCtrl;
      private LMVFrameRateCtrlLib.ILMVFrameRateCtrl lmvFrameRate;

      #region UI Events
      public MainForm()
      {
         InitializeComponent();
      }

      private void Form1_Load(object sender, EventArgs e)
      {
         //initialize the convert control
         InitConvertCtrl();
      }

      private void inputFileText_Click(object sender, EventArgs e)
      {
         //Get the name of the video file to convert
         OpenFileDialog dlg = new OpenFileDialog();
         dlg.Filter = "Video Files (*.avi;*.mpg;*.mpeg;*.mxf;*.mkv;*.ogg;*.wmv.*.asf;*.qt;*.mov;*.m2v;*.m1v;*.ts;*.m2ts;*.mp4;*.3gp;*.flv;*.f4v;video_ts.ifo;*.LBL)|*.avi;*.mpg;*.mpeg;*.mxf;*.mkv;*.ogg;*.wmv.*.asf;*.qt;*.mov;*.m2v;*.m1v;*.ts;*.m2ts;*.mp4;*.3gp;*.flv;*.f4v;video_ts.ifo;*.LBL|Windows Media Files (*.asf;*.wma;*.wmv)|*.asf;*.wma;*.wmv|DVD Files (video_ts.ifo)|video_ts.ifo|All Files (*.*)|*.*";
         if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
         {
            _inputFileText.Text = dlg.FileName;
            //once we have the video file, we need to use the MediaInfo to initialize our properties
            MediaInfo mi = new MediaInfo();
            mi.SourceFile = _inputFileText.Text;
            _endTimeNum.Maximum = (decimal)mi.SourceDuration;
            _endTimeNum.Value = (decimal)mi.SourceDuration;
            _frameRateNum.Maximum = (decimal)mi.VideoFrameRate;
            _frameRateNum.Value = (decimal)mi.VideoFrameRate;
            groupBox1.Enabled = true;
         }
      }

      private void outputFileText_Click(object sender, EventArgs e)
      {
         //Get the output file name to save the GIF
         SaveFileDialog dlg = new SaveFileDialog();
         dlg.Filter = "GIF | *.gif";
         if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
         {
            _outputFileText.Text = dlg.FileName;
         }
      }

      private void convertBtn_Click(object sender, EventArgs e)
      {
         //Do some error checking, make sure the output file is filled out and that the input file still exists
         if (string.IsNullOrWhiteSpace(_outputFileText.Text))
         {
            MessageBox.Show("Please input an output path!");
            return;
         }
         else if (string.IsNullOrWhiteSpace(_inputFileText.Text) || !System.IO.File.Exists(_inputFileText.Text))
         {
            MessageBox.Show("Input file doesn't exist!");
            return;
         }
         else if (System.IO.File.Exists(_outputFileText.Text)) //check if output file already exists, if it does prompt for deletion
         {
            DialogResult diag = MessageBox.Show("This file already exists, would you like to overwrite it?", "File Already Exists", MessageBoxButtons.YesNo);
            if (diag == DialogResult.Yes)
            {
               File.Delete(_outputFileText.Text);
            }
            else
               return;
         }

         //set the source and target files and the selection/end times
         convertCtrl.TargetFormat = TargetFormatType.StillImage;
         convertCtrl.TargetType = TargetObjectType.File;
         convertCtrl.TargetFile = _outputFileText.Text;
         convertCtrl.SourceFile = _inputFileText.Text;

         //set the selection start and end time to the UI specified times
         convertCtrl.SelectionStart = (double)_startTimeNum.Value;
         convertCtrl.SelectionEnd = (double)_endTimeNum.Value;

         //get the Still Image Writer filter and set the properties to save to a single multipage GIF
         LTStlImgWrtLib.ILTStlImgWrt stillImgWrt = (LTStlImgWrtLib.ILTStlImgWrt)convertCtrl.GetSubObject(ConvertObject.TargetFilter);
         stillImgWrt.SaveMultipage = true;
         stillImgWrt.FileTypeFormat = LTStlImgWrtLib.FileTypeConstants.SAVEFILE_GIF;
         stillImgWrt.BitsPerPixel = 8;
         stillImgWrt.PromptOverwrite = false;
         System.Runtime.InteropServices.Marshal.ReleaseComObject(stillImgWrt);

         //check to see if we want to specify a frame rate, if so, insert the filter
         if (_useFrameRateCheck.Checked)
            insertFrameRateFilter();

         //Check to see how many frames will be in the GIF and give a warning if more than 200
         int frames = (int)((convertCtrl.SelectionEnd - convertCtrl.SelectionStart) * (double)_frameRateNum.Value);
         if (frames > 200)
         {
            var dlg = MessageBox.Show("Warning!\nTotal frames in GIF will exceed 200. Do you wish to continue?", "Large Output Detected!", MessageBoxButtons.YesNo);
            if (dlg == System.Windows.Forms.DialogResult.No)
               return;
         }
         convertCtrl.StartConvert();

      }

      private void useFrameRateCheck_CheckedChanged(object sender, EventArgs e)
      {
         _frameRateNum.Enabled = _useFrameRateCheck.Checked;
      }
      #endregion UI

      private void insertFrameRateFilter()
      {
         //add the frame rate filter and get the object and set the properties
         convertCtrl.SelectedVideoProcessors.Add(convertCtrl.VideoProcessors.FrameRateControl);
         lmvFrameRate = (LMVFrameRateCtrlLib.ILMVFrameRateCtrl)convertCtrl.GetSubObject(ConvertObject.SelVideoProcessor);
         lmvFrameRate.Enable = true;
         lmvFrameRate.FrameRate = (double)_frameRateNum.Value;
         System.Runtime.InteropServices.Marshal.ReleaseComObject(lmvFrameRate);
      }

      void InitConvertCtrl()
      {
         //initialize the Convert Control and subscribe to events
         convertCtrl = new ConvertCtrl();
         convertCtrl.Progress += new ProgressEventHandler(convertCtrl_Progress);
         convertCtrl.Complete += new EventHandler(convertCtrl_Complete);
      }

      void convertCtrl_Complete(object sender, EventArgs e)
      {
         //Once we are done, open up the directory and reset the progress bar and remove the frame rate filter
         MessageBox.Show("Done!");
         var folderName = Path.GetDirectoryName(_outputFileText.Text);
         Process.Start(folderName);
         _progressBar.Value = 0;
         convertCtrl.SelectedVideoProcessors.Remove(convertCtrl.VideoProcessors.FrameRateControl);
      }

      void convertCtrl_Progress(object sender, ProgressEventArgs e)
      {
         _progressBar.Value = e.percent;
      }
   }
}
